@@ -22,6 +22,8 @@ test_that("general checks in ammi plot function properly", {
                "Invalid value provided for primAxis")
   expect_error(plot(geAmmi, plotType = "AMMI2", secAxis = "PCC"),
                "Invalid value provided for secAxis")
+  expect_error(plot(geAmmi, rotatePC = 1),
+               "rotatePC should be NULL or a character string.")
 })
 
 p0_1 <- plot(geAmmi)
@@ -85,10 +87,10 @@ test_that("AMMI plot sizeGeno functions properly", {
   geoms1_1 <- sapply(p1_1$layers, function(x) class(x$geom)[1])
   geoms0_2 <- sapply(p0_2$layers, function(x) class(x$geom)[1])
   geoms1_2 <- sapply(p1_2$layers, function(x) class(x$geom)[1])
-  expect_equal(geoms1_1[geoms0_1 == "GeomPoint"], "GeomText")
+  expect_equivalent(geoms1_1[geoms0_1 == "GeomPoint"], "GeomText")
   dat1_1 <- p1_1$layers[geoms0_1 == "GeomPoint"][[1]]$data
   expect_equal(unique(dat1_1[dat1_1[["type"]] == "geno", ".size"]), 5)
-  expect_equal(geoms1_2[geoms0_2 == "GeomPoint"], "GeomText")
+  expect_equivalent(geoms1_2[geoms0_2 == "GeomPoint"], "GeomText")
   dat1_2 <- p1_2$layers[geoms0_2 == "GeomPoint"][[1]]$data
   expect_equal(unique(dat1_1[dat1_1[["type"]] == "geno", ".size"]), 5)
 })
@@ -314,6 +316,14 @@ test_that("AMMI plot gives correct output types when byYear = TRUE", {
   expect_is(p2[[2]], "ggplot")
 })
 
+test_that("AMMI plot rotatePC functions properly", {
+  p1_env <- plot(geAmmi, rotatePC = "E1")
+  p1_geno <- plot(geAmmi, rotatePC = "G1")
+  ## Rotation should rotate E1/G1 in such a way that y coordinate becomes 0.
+  expect_equal(p1_env[["data"]]["E1", "y"], 0)
+  expect_equal(p1_geno[["data"]]["G1", "y"], 0)
+})
+
 ## Finlay Wilkinson
 
 geFw <- gxeFw(TD = testTD, trait = "t1", maxIter = 30)
@@ -337,21 +347,22 @@ test_that("FW plot gives correct output types", {
 
 test_that("Option colorGenoBy in scatter plot functions correctly", {
   p1 <- plot(geFw, colorGenoBy = "family")
-  expect_equal(p1[[1]]$labels$colour, "family")
-  expect_equal(p1[[2]]$labels$colour, "family")
-  expect_equal(p1[[3]]$labels$colour, "family")
+  ggplot2::get_labs(p1[[1]])
+  expect_equal(ggplot2::get_labs(p1[[1]])$colour, "family")
+  expect_equal(ggplot2::get_labs(p1[[2]])$colour, "family")
+  expect_equal(ggplot2::get_labs(p1[[3]])$colour, "family")
 })
 
 test_that("Option colorGenoBy in line plot functions correctly", {
   p1 <- plot(geFw, plotType = "line", colorGenoBy = "family")
-  expect_equal(p1$labels$colour, "family")
+  expect_equal(ggplot2::get_labs(p1)$colour, "family")
   ## With coloring plot should have a legend explicitly defined.
   expect_equal(p1$theme$legend.position, "right")
 })
 
 test_that("Option colorGenoBy in scatterFit plot functions correctly", {
   p1 <- plot(geFw, plotType = "scatterFit", colorGenoBy = "family")
-  expect_equal(p1$labels$colour, "family")
+  expect_equal(ggplot2::get_labs(p1)$colour, "family")
 })
 
 test_that("option order in FW line plot functions properly", {
@@ -402,7 +413,7 @@ test_that("VarCov plot gives correct output types", {
   p <- plot(geVarCov)
   geoms <- sapply(p$layers, function(x) class(x$geom)[1])
   expect_is(p, "ggplot")
-  expect_equal(geoms, "GeomTile")
+  expect_equivalent(geoms, "GeomTile")
 })
 
 ## melting data in the plot function caused an error when trials have a
